

unit Parser;

// parser
// - this file is code-generated

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
	Nodes, Scanner, Node;

type

	TreeIterator = procedure(node: TNode) of object;


	TParser = class
	private
		scan: TScanner;
		nodeRoot: TNode;
		syntaxErrorMsg: TStringList;

	public
		constructor Create;
		destructor Destroy; override;
		property Root: TNode read nodeRoot;
		procedure Parse(AMemo: TMemo);
		property SyntaxErrorMessage: TStringList read syntaxErrorMsg;

	private
		



// productions:
      function S: TSNode;
      function StmtList: TStmtListNode;
      function Stmt: TStmtNode;
      function AssignmentStmt: TAssignmentStmtNode;
      function AddExpr: TAddExprNode;
      function SubExpr: TSubExprNode;
      function TimesExpr: TTimesExprNode;
      function DivideExpr: TDivideExprNode;
      function SimpleExpr: TSimpleExprNode;
      function ShowStmt: TShowStmtNode;
      function Identifier: TIdentifierNode;
      function Number: TNumberNode;

// terminators:
//   tokEquals
//   tokLParen
//   tokRParen
//   tokPlus
//   tokMinus
//   tokTimes
//   tokDivide
//   tokIdent
//   tokNumber
//   tokShowKeyword
//   tokEof



	end;

	ParseException = class(Exception);
	ParseSyntaxErrorException = class(ParseException);
		


// =======================================================

implementation

{$HINTS OFF}




// ----------------------------------------------------------
// S
function TParser.S: TSNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  node1 := StmtList();
  scan.MatchScan(tokEof);
  result := TSNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// StmtList
function TParser.StmtList: TStmtListNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  node1 := Stmt();
  case scan.Token of
    tokIdent, tokShowKeyword:
      begin
        node2 := StmtList();
      end;
  end;
  result := TStmtListNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// Stmt
function TParser.Stmt: TStmtNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  case scan.Token of
    tokIdent:
      begin
        node1 := AssignmentStmt();
      end;
    tokShowKeyword:
      begin
        node1 := ShowStmt();
      end;
    else
      scan.Match(tokIdent);
  end;
  result := TStmtNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// AssignmentStmt
function TParser.AssignmentStmt: TAssignmentStmtNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  node1 := Identifier();
  scan.MatchScan(tokEquals);
  node2 := AddExpr();
  result := TAssignmentStmtNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// AddExpr
function TParser.AddExpr: TAddExprNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  node1 := SubExpr();
  case scan.Token of
    tokPlus:
      begin
        scan.Scan;
        node2 := AddExpr();
      end;
  end;
  result := TAddExprNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// SubExpr
function TParser.SubExpr: TSubExprNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  node1 := TimesExpr();
  case scan.Token of
    tokMinus:
      begin
        scan.Scan;
        node2 := SubExpr();
      end;
  end;
  result := TSubExprNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// TimesExpr
function TParser.TimesExpr: TTimesExprNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  node1 := DivideExpr();
  case scan.Token of
    tokTimes:
      begin
        scan.Scan;
        node2 := TimesExpr();
      end;
  end;
  result := TTimesExprNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// DivideExpr
function TParser.DivideExpr: TDivideExprNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  node1 := SimpleExpr();
  case scan.Token of
    tokDivide:
      begin
        scan.Scan;
        node2 := DivideExpr();
      end;
  end;
  result := TDivideExprNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// SimpleExpr
function TParser.SimpleExpr: TSimpleExprNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  case scan.Token of
    tokNumber:
      begin
        node1 := Number();
      end;
    tokIdent:
      begin
        node1 := Identifier();
      end;
    else
      scan.Match(tokNumber);
  end;
  result := TSimpleExprNode.Create(node1, node2);
end;

// ----------------------------------------------------------
// ShowStmt
function TParser.ShowStmt: TShowStmtNode;
var
  node1, node2: TNode;
begin
  node1 := nil;
  node2 := nil;
  scan.MatchScan(tokShowKeyword);
  node1 := Identifier();
  result := TShowStmtNode.Create(node1, node2);
end;



{$HINTS ON}


// --------------------------------------------------------------------------
// class constructor
constructor TParser.Create;
begin
	scan := TScanner.Create;
	syntaxErrorMsg := TStringList.Create;
	nodeRoot := nil;
end;

// --------------------------------------------------------------------------
// class destructor
destructor TParser.Destroy;
begin
	syntaxErrorMsg.Free;
	if assigned(nodeRoot) then
		nodeRoot.free;
	scan.free;
end;

// --------------------------------------------------------------------------
// parse a file
procedure TParser.Parse(AMemo: TMemo);
var
	sPointer: String;
begin
	scan.SetMemo(AMemo);
	try
		scan.Scan;
		nodeRoot := S;
	except
		syntaxErrorMsg.Clear;
		syntaxErrorMsg.Add('syntax error on line ' + IntToStr(scan.CurrentLineCount) + ':');
		syntaxErrorMsg.Add('');
		sPointer := StringOfChar(' ', scan.CurrentPosition) +
			StringOfChar('^', Length(scan.CurrentLine) - scan.CurrentPosition);
		syntaxErrorMsg.Add(scan.FullLine());
		syntaxErrorMsg.Add(sPointer );
		raise ParseSyntaxErrorException.Create('');
	end;
end;

// --------------------------------------------------------------------------
// identifier (eg. variable name)
function TParser.Identifier: TIdentifierNode;
var
	sValue: String;
begin
	sValue := scan.StringValue;
	scan.MatchScan(tokIdent);
	result := TIdentifierNode.Create(sValue);
end;

// --------------------------------------------------------------------------
// number value
function TParser.Number: TNumberNode;
var
	iValue: Integer;
begin
	iValue := scan.NumberValue;
	scan.MatchScan(tokNumber);
	result := TNumberNode.Create(iValue);
end;



end.


